use [DataScience]
go

/*

-- create the log for all symbols

drop table if exists dbo.log_for_buy_at_ema_10_gt_50

-- create dbo.log_for_buy_at_ema_10_gt_50
select 
 date
,symbol
,[open]
,[close]
,ema_10_lag_1
,ema_50_lag_1
,ema_10
,ema_50

-- set buy and sell signals
,case
	when ((ema_10_lag_1 <= ema_50_lag_1) and  (ema_10 > ema_50)) then 'buy signal'
	when ((ema_10_lag_1 >= ema_50_lag_1) and (ema_50 > ema_10)) then 'sell signal'
	else null
 end buy_sell_signal

-- set buy and sell prices
,case
	when ((ema_10_lag_1 <= ema_50_lag_1) and  (ema_10 > ema_50)) then open_lead_1
	when ((ema_10_lag_1 >= ema_50_lag_1) and (ema_50 > ema_10)) then open_lead_1
	else null
 end buy_sell_price

into dbo.log_for_buy_at_ema_10_gt_50

from
(
-- source rows for buy/sell actions
select 
 stooq_prices.* 
,denormalized_emas.ema_10
,denormalized_emas.ema_50
,denormalized_emas.ema_200
,lag(denormalized_emas.ema_10,1) over (partition by stooq_prices.symbol 
		order by denormalized_emas.date) ema_10_lag_1
,lag(denormalized_emas.ema_50,1) over (partition by stooq_prices.symbol 
		order by denormalized_emas.date) ema_50_lag_1
,lead(stooq_prices.[open],1) over(partition by stooq_prices.symbol 
		order by  denormalized_emas.date) open_lead_1

from stooq_prices
inner join denormalized_emas
on stooq_prices.date = denormalized_emas.date
and stooq_prices.symbol = denormalized_emas.symbol
) for_buy_sell_log_for_ema_10_vs_50

--select * from dbo.log_for_buy_at_ema_10_gt_50

*/

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------

/*

-- Cleaning the log for APPL to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'AAPL'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

-- echo two resource tables
--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'AAPL'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100
	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles

insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

/*

-- Cleaning the log for GOOGL to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'GOOGL'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'GOOGL'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100

	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles


insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

/*

-- Cleaning the log for MSFT to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'MSFT'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'MSFT'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100

	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles


insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

/*

-- Cleaning the log for SPXL to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'SPXL'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'SPXL'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100

	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles


insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

/*

-- Cleaning the log for TQQQ to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'TQQQ'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'TQQQ'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100

	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles


insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------

/*

-- Cleaning the log for UDOW to compute a single buy_sell_date for
-- for the start and end of successive cycles in the log

-- declarations
declare 
	@symbol nvarchar(10) = 'UDOW'
   ,@first_buy_signal_date date
   ,@first_sell_signal_date date
   ,@prior_buy_signal_date date
   ,@prior_sell_signal_date date
   ,@next_buy_signal_date date
   ,@next_sell_signal_date date

-- create and populate a temp log for a symbol 
-- with buy_sell_signal dates (next_date)
drop table if exists dbo.symbol_temp_log

select [date]
      ,[symbol]
      ,[open]
      ,[close]
      ,[buy_sell_signal]
      ,[buy_sell_price]
	  ,lead(date,1) over (partition by symbol order by date) next_date
into dbo.symbol_temp_log
from   [DataScience].[dbo].[log_for_buy_at_ema_10_gt_50]
where symbol = @symbol

--select * from dbo.log_for_buy_at_ema_10_gt_50
--select * from dbo.symbol_temp_log


-- create a fresh version of a table for buy_sell 
-- cycle dates with a symbol

drop table if exists dbo.symbol_buy_sell_cycle_date

create table dbo.symbol_buy_sell_cycle_date(
	 [symbol] [nvarchar](10) not NULL
	,[buy_sell_cycle_date] [date] NOT NULL,
)


-- compute @first_buy_signal_date and @first_sell_signal_date
-- and insert into dbo.symbol_buy_sell_cycle_date

set @first_buy_signal_date = (select min(next_date) 
from symbol_temp_log
where buy_sell_signal = 'buy signal')

set @first_sell_signal_date = (select min(next_date)
from symbol_temp_log
where buy_sell_signal = 'sell signal')

set @prior_buy_signal_date = @first_buy_signal_date
set @prior_sell_signal_date = @first_sell_signal_date

insert dbo.symbol_buy_sell_cycle_date
select *
from
(
select @symbol [symbol], @first_buy_signal_date [buy_sell_cycle_date]
union
select @symbol [symbol], @first_sell_signal_date [buy_sell_cycle_date]
) buy_sell_cycle_dates


-- after search for @first_buy_signal_date and @first_sell_signal_date
-- search iteratively for @next_buy_signal_date and @next_sell_signal_date

Next_Buy_Sell_Date_Pair:
print 'got to label'

set @next_buy_signal_date = (select min(next_date) next_buy_signal_date
from symbol_temp_log 
where buy_sell_signal = 'buy signal' and next_date > @prior_sell_signal_date)
--select @next_buy_signal_date [@next_buy_signal_date]

set @next_sell_signal_date = (select min(next_date) next_sell_signal_date 
from symbol_temp_log
where buy_sell_signal = 'sell signal' and next_date > @next_buy_signal_date)
--select @next_sell_signal_date [@next_sell_signal_date]

set @prior_buy_signal_date = @next_buy_signal_date
set @prior_sell_signal_date = @next_sell_signal_date



-- cease searching interatively when at least one null value is returned 
-- when no null returned insert cycle dates in symbol_buy_sell_cycle_date table
if @next_buy_signal_date is not null and @next_sell_signal_date is not null
begin

	insert dbo.symbol_buy_sell_cycle_date
	select *
	from
	(
	select @symbol [symbol], @next_buy_signal_date [buy_sell_cycle_date]
	union
	select @symbol [symbol], @next_sell_signal_date [buy_sell_cycle_date]
	) buy_sell_cycle_dates

	goto Next_Buy_Sell_Date_Pair

end
else 
begin

	select * from symbol_buy_sell_cycle_date

	return

end

--------------------------------------------------------------------------------------

-- compute price change and percentage price change per buy_sell cycle

-- drop table if exists requires a preceding go when dropping a temp table
-- so that the statement runs at the top of a batch
go
drop table if exists #percent_change_and_percentage_change_per_cycle

-- have to repeat declaration for @symbol in this batch
declare	@symbol nvarchar(10) = 'UDOW'

select 
	 *
	,case
		when buy_sell_signal = 'sell signal' then
			buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date))  
	 end price_change
	,case
		when buy_sell_signal = 'sell signal' then
			(buy_sell_price
			- (lag([buy_sell_price],1) over(order by buy_sell_cycle_date)))/
			(lag([buy_sell_price],1) over(order by buy_sell_cycle_date))*100

	 end percentage_price_change
into #percent_change_and_percentage_change_per_cycle
from
(
-- join of buy_sell_price values from the symbol_temp_log table
-- with buy_sell_cycle_date values from symbol_buy_sell_cycle_date table
-- returns buy and sell prices for each buy_sell cycle
select 
 symbol_temp_log.date
,symbol_temp_log.symbol
,symbol_temp_log.buy_sell_signal
,symbol_temp_log.buy_sell_price
,symbol_temp_log.next_date
,symbol_buy_sell_cycle_date.buy_sell_cycle_date

from dbo.symbol_temp_log
inner join dbo.symbol_buy_sell_cycle_date on
symbol_temp_log.next_date = symbol_buy_sell_cycle_date.buy_sell_cycle_date
) buy_sell_prices_for_price_change_per_cycle

declare 
 @#_of_winners float = 0.0
,@#_of_cycles float = 0.0
,@avg_price_change float
,@avg_percentage_price_change float


select 
 @#_of_cycles = cast(count(*) as real)/2
from #percent_change_and_percentage_change_per_cycle

select @#_of_winners = sum(for_#percentage_change_per_cycle)
from
(
select
 case
	when percentage_price_change > 0 then 1
	else 0
 end for_#percentage_change_per_cycle
from #percent_change_and_percentage_change_per_cycle
) for_#_of_winning_cycles


insert into dbo.symbol_model_cycles
select 
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,@#_of_cycles [# of cycles]
,@#_of_winners [# of winning cycles]
,round((@#_of_winners/@#_of_cycles)*100,2) [percentage winning cycles]


insert into dbo.symbol_model_price_changes
select
 @symbol [symbol]
,'log_for_buy_at_ema_10_gt_50' [log name]
,cast(count(*) as real)/2 [# of cycles]
,avg(price_change) [average price change per cycle]
,sum(price_change) [total price change per share]
,round(avg(percentage_price_change),2) [average percentage price change per cycle]
from #percent_change_and_percentage_change_per_cycle


select * from #percent_change_and_percentage_change_per_cycle


*/

--------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------
